function [OutCellArray, OutConfig]=ApplyArbitraryFunction(InCellArray, InConfig)
% APPLY ARBITRARY FUNCTION applies a user-specified function to a single dataset.
%
% This function allows users to perform arbitrary calculations on a single
% dataset. The calculations are limited only to the functions availble
% within MATLAB.
%
% To specify where the data should be applied, use the key word 'INPUT'.
% For instance, to compute the sine for each pixel in a .map datatype, the
% function would be entered as:
%                           sin(INPUT)
%
% The only requirement on this funcion is that the output data have the
% same datatype as the input. This module does not modify header parameters
% or convert datatypes. Because arbitrary function can produce arbitrary
% output, it may be necessary to further modify the header information
% manually. For non-trivial calculations, or conversion between data types,
% it is recommended to write a separate module instead of trying to use
% this module.
%
% Input: coordinates, freqmap, layermap, map, mask, or zernike
% Output: same as input
%
% Author: Greg Smith
% Email: gsmith@optics.arizona.edu
%
% Version history:
% 10/8/2012 - Version 1 - Greg Smith - initial implementation

% use input as default output
OutConfig = InConfig;
OutCellArray = InCellArray;

% create basic GUI to query user for function to apply.
prompt = ['Enter a function to be applied to your data. ',...
    'Use the word ''INPUT'' to denote the input dataset. \newline',...
    'note: It is assumed the output is the same data type as the input.\newline\newline',...
    'Example:  function = ''sin(INPUT)'' will compute the sine of every pixel in a map.\newline'];
name='Function Input';
numlines=1;
defaultanswer=InConfig(strcmpi(InConfig, 'function'),2);
options.Resize='off';
options.WindowStyle='modal';
options.Interpreter='tex';
answer=inputdlg(prompt,name,numlines,defaultanswer,options);

% get old data     
originalData = InCellArray{1}.data;

% apply calculation to the original data
if isempty(answer)
    % calculation aborted
    OutCellArray = {};
else
    evalString = regexprep(answer{1}, 'INPUT', 'originalData');
    newData = eval(evalString);
    
    % update data output
    % throw warning message if the data dimensions have changed
    if size(originalData) == size(newData)
        OutCellArray{1}.data = newData;
        OutConfig{strcmpi(OutConfig, 'function'),2} = evalString;
    else
        warndlg(['Warning! The output has different dimensions than the input. ',...
            'Please change your function and try again. Conversion is halted.'],...
            'Dimension mismatch','modal');
        OutCellArray = {};
    end
end

end