function [OutCellArray,Outconfig]=MapMasktool(InCellArray, Inconfig)
% The map masking tool masks a dataset.  The map can be specified in
% several ways. A file can be specified for the module to use using the GUI
% or the config editor. Another option is to specify an OD and ID for an
% annular mask. A third option is to run this module after selecting both a
% map and a mask. In this case, the mask will be applied to the map and the
% masked map will be returned.
%
% Inputs: Map or (Map & Mask)
% Outputs: Map
%
% Author: Benjamin Lewis
% Email: blewis@email.arizona.edu
%
% Version history:
% 8/10/10 - Version 1
% 4/5/2012 - Version 1.0.1 - G.S. 
%                          - fixed bug to work with new SAGUARO kernel
%                          - fix "trivial" mLint warnings

% Check if units match, if they don't, return.
% For input make sure the OD includes the original units.
% Warn them that the filter may be oval shapped depending on data dim.

Outconfig=Inconfig;

OutCellArray{1}=InCellArray{1};

% Read header
% Radius=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Half Width (mm)'),2};
% pts=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Data Points'),2};

% New header code:
SizeX  = str2double(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X size'),2});
SizeY  = str2double(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y size'),2});

ptsX = size(InCellArray{1}.data, 2);
ptsY = size(InCellArray{1}.data, 1);

if length(InCellArray)==2 %Both map and mask specified
    mask=InCellArray{2}.data;
    mask(mask==0)=NaN;
    data = InCellArray{1}.data;
    if ~all(size(mask)==size(data))
        msgbox('Mask must have same number of pixels as map. No masking will be performed.','Incompatible Inputs','warn','modal');
        OutCellArray = [];
        Outconfig = [];
        return;
    end
    OutCellArray{1}.data=data.*mask;
else %Either using a file, or specifying OD and ID
    OD=Inconfig{strcmpi(Inconfig, 'OD (mm)'),2};
    
    ID=Inconfig{strcmpi(Inconfig, 'ID (mm)'),2};
    
    TempUseFile=Inconfig{strcmpi(Inconfig, 'Use File'),2};
    
    MapFilename=Inconfig{strcmpi(Inconfig, 'Map Filename'),2};    

    % x=linspace(-1, 1, str2num(pts)); %Compute radial cordinate
    % x=meshgrid(x);
    % [~, rho] = cart2pol(x, -1*x');  
    % check map statistics module
    
    % rectangular coordinates
    x = linspace(-1, 1, ptsX)*SizeX/2;
    y = linspace(-1, 1, ptsY)*SizeY/2;

    [X, Y] = meshgrid(x, y);
    Radius = sqrt(X.^2+Y.^2);
    
    [~, rho] = cart2pol(X, Y);
    
    %GUI intialization
    fig = figure('Visible','off','Position',[200,200,300,345], 'resize', 'off', 'MenuBar', 'none', 'ToolBar','none');
    set(fig,'Name','SAGUARO Mask');
    movegui(fig,'center');
    
    mapax=axes('Units','Pixels','Position',[48,58,220,220]);
    
    IDtext=uicontrol('Style','text','String','Enter ID (mm):', 'Position',[170,285,75,15]);
    ODtext=uicontrol('Style','text','String','Enter OD (mm):', 'Position',[20,285,75,15]);    
    UseFile=uicontrol('Style','checkbox','String','Use Mask from File:', 'Position',[20,330,130,15],'Callback',{@Usefile_callback}, 'Value', str2num(TempUseFile));
        
    OK=uicontrol('String','OK', 'Position',[15,20,45,20], 'Callback', {@OK_callback});
    MaskBrowse=uicontrol('String','Browse for Mask', 'Position',[20,305,90,20], 'Callback', {@Browse_callback});
    
    IDedit=uicontrol('Style','edit','String','', 'Position',[260,285,45,15], 'Callback',{@ID_callback});
    ODedit=uicontrol('Style','edit','String','', 'Position',[110,285,45,15], 'Callback',{@OD_callback});
    Fileedit=uicontrol('Style','edit','String','', 'Position',[165,330,140,15], 'Callback',{@Fileedit_callback});
    
    set(IDedit, 'String', ID);
    set(ODedit, 'String', OD);
    set(Fileedit, 'String', MapFilename); 
    
    mask=zeros(ptsX, ptsY);
    
    ID=str2double(ID);
    
    OD=str2double(OD);
    
    %Load a file if we are using one
    if ~get(UseFile,'value')
        set([IDedit, ODedit], 'Enable', 'on');
        %display(size(rho));
        %display(size(Radius));
        %display(size(mask));
        
        mask(rho<=OD/2&rho>=ID/2)=1;
    else
        set([IDedit, ODedit], 'Enable', 'off');
        temp=SAGUARO_dataset;
        temp.load(MapFilename);
        if ~isempty(temp.data)
            mask=temp.data;
        else
            error_h=errordlg(sprintf('Mask File Not Found\n Map Mask Tool will not load file'), 'MapMaskTool Error', 'modal');
            uiwait(error_h);
            set(UseFile,'value', 0);
            set([IDedit, ODedit], 'Enable', 'on');
            mask(rho<=OD/2&rho>=ID/2)=1;
        end
    end
    
    TempUseFile=get(UseFile,'value');
    
    %Plotting
    axes(mapax);
    imagesc(mask);
    
    set(fig, 'Visible', 'on');
    
    uiwait(fig); %Wait for figure close
    
    if ishandle(fig), delete(fig); end
    
    if any(size(mask)~=size(OutCellArray{1}.data))
        errordlg('Mask Size Mismatch', 'MapMaskTool Error', 'modal');
        return;
    end
    
    if ~isempty(num2str(ID))
        Outconfig{strcmpi(Outconfig, 'ID (mm)'),2}=num2str(ID);
    end
    
    if ~isempty(num2str(OD))
        Outconfig{strcmpi(Outconfig, 'OD (mm)'),2}=num2str(OD);
    end
    
    Outconfig{strcmpi(Outconfig, 'Map Filename'),2}=MapFilename;
    
    Outconfig{strcmpi(Outconfig, 'Use File'),2}=num2str(TempUseFile);
    
    if max(mask)==0, mask=ones(ptsX, ptsY); end %Set up mask
    
    mask(mask==0)=NaN;
    
    OutCellArray{1}.data=InCellArray{1}.data.*mask; %Mask data and return
end


    function ID_callback(hObject, ~) %ID value changed
        if ~isempty(get(hObject, 'String'))&&isnumeric(str2num(get(hObject, 'String')))
            ID=str2double(get(hObject, 'String'));
        else
            ID=0;
            set(hObject, 'String', num2str(ID));
        end
        mask=zeros(ptsX, ptsY);
        mask(rho<=OD/2&rho>=ID/2)=1;
        axes(mapax);
        imagesc(mask);
    end

    function OD_callback(hObject, ~) %OD changed
        if ~isempty(get(hObject, 'String'))&&isnumeric(str2num(get(hObject, 'String')))
            OD=str2double(get(hObject, 'String'));
        else
            set(hObject, 'String', num2str(OD));
        end
        mask=zeros(ptsX, ptsY);
        mask(rho<=OD/2&rho>=ID/2)=1;
        axes(mapax);
        imagesc(mask);        
    end

    function Usefile_callback(~, ~) %User either selected or deselected usefile checkbox
        if ~get(UseFile,'value') %check if usefile, if not, compute mask
            set([IDedit, ODedit], 'Enable', 'on');
            mask=zeros(ptsX, ptsY);
            mask(rho<=OD/2&rho>=ID/2)=1;
            axes(mapax);
            imagesc(mask);            
        else %using a file
            set([IDedit, ODedit], 'Enable', 'off');
            temp=SAGUARO_dataset;
            temp.load(MapFilename);
            if ~isempty(temp.data)
                mask=temp.data;
                axes(mapax);
                imagesc(mask);
            else
                errordlg('Mask  File Not Found', 'MapMaskTool Error', 'modal');
                mask=zeros(ptsX, ptsY);
                mask(rho<=OD/2&rho>=ID/2)=1;
                axes(mapax);
                imagesc(mask);
                set(UseFile,'value', 0);
                set([IDedit, ODedit], 'Enable', 'on');                
            end
        end
        TempUseFile=get(UseFile,'value');
    end

    function Fileedit_callback(hObject, eventdata) %file being used has changed
        MapFilename=get(Fileedit,'string');
        set(UseFile,'value', 1);
        Usefile_callback(hObject, eventdata);         
    end

    function Browse_callback(~, ~) %browsing for a file
        [maskfile, pathname]=uigetfile('*.*','Select a file to use as a mask', 'MultiSelect','off');
        if maskfile==0
            return;
        end
        MapFilename=fullfile(pathname, maskfile);
        set(Fileedit,'string', MapFilename);
        set([IDedit, ODedit], 'Enable', 'off');
        temp=SAGUARO_dataset;
        temp.load(MapFilename); %load mask
        if ~isempty(temp.data)
            mask=temp.data;
            axes(mapax);
            imagesc(mask);
            set(UseFile,'value', 1);
            set([IDedit, ODedit], 'Enable', 'off');
        else
            errordlg('Mask File Not Found', 'MapMaskTool Error', 'modal');
            mask=zeros(ptsX, ptsY);
            mask(rho<=OD/2&rho>=ID/2)=1;
            axes(mapax);
            imagesc(mask);
            set(UseFile,'value', 0);
            set([IDedit, ODedit], 'Enable', 'on');        
        end
        TempUseFile=get(UseFile,'value');
    end

    function OK_callback(~, ~) %on user ok, close figure to unblock execution
        if ishandle(fig), delete(fig); end
    end
end


