function [OutCellArray,Outconfig]=MaskBadData(InCellArray, Inconfig)
%This module provides a GUI to allow the user to interactively select bad
%or invalid data on a map, and returns NaN in place of the selected data,
%as well as the mask used to exclude the data.
%
%Inputs: Map
%Outputs: Map & Mask
%
%Author: Sara (Landau) Lampen
%Email: slandau@optics.arizona.edu
%
% Version history:
% 10/19/10 - Version 2
% 08/20/11 - Version 2.1 (Improved user commands)
% 02/15/12 - Version 2.2 Updated header

%Header to interface with SAGUARO
Outconfig=Inconfig;

try
    OutCellArray{1}=InCellArray{1};
    
    % New header code:
    ptsX = size(InCellArray{1}.data, 2);
    ptsY = size(InCellArray{1}.data, 1);
    
    if strcmpi(InCellArray{1}.type,'map')
        SizeX  = str2double(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X size'),2});
        SizeY  = str2double(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y size'),2});
    else
        SizeX = ptsX;
        SizeY = ptsY;
    end

    RadiusX = SizeX/2;
    RadiusY = SizeY/2;
    
    dmap=InCellArray{1}.data;
    x_axis=linspace(-RadiusX,RadiusX,ptsX);
    y_axis=linspace(-RadiusY,RadiusY,ptsY);
    
    fig=figure;
    
    %Plot current image
    imagesc(x_axis,y_axis,dmap,'alphadata',~isnan(dmap));
    
    current_mask=ones(size(dmap));
    temp_data=dmap;
    loop_index=1;
    %This while loop allows the user to continue to select areas of the map to
    %exclude using either an ellipsoidal mask or a polygonal mask.
    Qmore_masking='Y';
    while Qmore_masking=='Y' || Qmore_masking=='y'
        
        choice = questdlg('Choose a mask type or click Done to finish:', ...
            'Mask Type', ...
            'Ellipse','Polygon','Done','Done');
        
        if strcmp(choice,'Ellipse')|| strcmp(choice,'Polygon')
            add_remove = questdlg('Would you like to remove data from inside or outside the selected region?', ...
                'Outside or Inside', ...
                'Outside','Inside','Inside');
        end
        
        if strcmp(add_remove,'Outside')
            plot_data=dmap;
            qroi=plot_data(~current_mask);
            qroi_nan=isnan(qroi);
            qmean=mean(qroi(~qroi_nan));
            qstd=std(qroi(~qroi_nan));
            
            qthreshold=(abs(qroi)>=2*qstd);
            qroi(qthreshold)=qmean;
            plot_data(~current_mask)=qroi-qmean/2;
            %plot_data(~current_mask)=qroi-qmean*10;
            figure(fig)
            imagesc(x_axis,y_axis,plot_data,'alphadata',~isnan(plot_data));
        end
        
        switch choice
            case 'Ellipse'
                title('Select ellipsoidal ROI, then Double Click on a vertex to finish mask')
                h = imellipse;
                p=wait(h);
                %pause
                BW = createMask(h);
            case 'Polygon'
                title('Select polygonal ROI to remove, then Double Click on a vertex to finish mask')
                BW = roipoly;
            case 'Done'
                Qmore_masking='n';
                break
        end
        
        if strcmp(add_remove,'Outside')&&loop_index~=1
            current_mask=BW|current_mask;
        elseif strcmp(add_remove,'Outside')&&loop_index==1
            current_mask=BW;
        else
            current_mask=~BW&current_mask;
        end
        
        %Updates mask with new exclusions and updates plot
        temp_data=dmap;
        temp_data(~current_mask)=NaN;
        figure(fig)
        imagesc(x_axis,y_axis,temp_data,'alphadata',~isnan(temp_data));
        title('Current Masking')
        
        loop_index=loop_index+1;
    end
    
    export_data=dmap;
    export_data(~current_mask)=NaN;
    %Footer to interface with SAGUARO
    OutCellArray{1}.data=export_data;
    
    % In first cell modify the data
    % In second cell return the mask
    
    %Export Mask
    if ~strcmpi(InCellArray{1}.type,'mask')
        export_cell=struct;
        export_cell.type='mask';
        export_cell.modulecount=0;
        export_cell.data=double(current_mask);
        export_cell.data(export_cell.data == 0) = NaN;
        export_cell.header = {};
        export_cell.header(1,:) = {'Date Created', datestr(now, 'mm/dd/yy HH:MM PM')};
        OutCellArray{2}=export_cell;
    end
    
    %Error checking : If the user closes the window, the program returns to
    %SAGUARO
catch ex
    if strcmp(ex.identifier,'MATLAB:minrhs') && strcmp(ex.message,'Not enough input arguments.')
        return
    elseif strcmp(ex.identifier,'MATLAB:andOrXor:sizeMismatch') && strcmp(ex.message,'Inputs must have the same size.')
        return
        %     elseif strcmp(ex.identifier,'MATLAB:UndefinedFunction') && strcmp(ex.message,['Undefined function or variable ','''','BW','''','.'])
    elseif strcmp(ex.identifier,'MATLAB:UndefinedFunction')
        close(fig);
        return
    else
        rethrow(ex)
    end
end

close(fig);